package meerkat.bulletinboard.sqlserver;

import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;

import meerkat.protobuf.Voting.MessageFilterList;
import meerkat.rest.Constants;
import meerkat.bulletinboard.sqlserver.BulletinBoardSQLServer;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.Voting.BulletinBoardMessage;
import meerkat.protobuf.Voting.BulletinBoardMessageList;


@Path("/SQLServer")
public class SQLiteBulletinBoardServer extends BulletinBoardSQLServer {

	protected static final int TIMEOUT = 20;
	
	/**
	 * This procedure initializes:
	 * 1. The database connection
	 * 2. The database tables (if they do not yet exist).
	 */
	@PostConstruct
	@Override
	public void init() throws CommunicationException {
		
		try{
			
			connection = DriverManager.getConnection("jdbc:sqlite:local-instances/meerkat.db");
			Statement statement = connection.createStatement();
			statement.setQueryTimeout(TIMEOUT);
	
			statement.executeUpdate("CREATE TABLE IF NOT EXISTS MsgTable (EntryNum INTEGER PRIMARY KEY, MsgId BLOB UNIQUE, Msg BLOB, SignerId BLOB)");
			statement.executeUpdate("CREATE TABLE IF NOT EXISTS TagTable (TagId INTEGER PRIMARY KEY, Tag varchar(50) UNIQUE)");
			statement.executeUpdate("CREATE TABLE IF NOT EXISTS MsgTagTable (MsgId BLOB, TagId INTEGER, FOREIGN KEY (MsgId) REFERENCES MsgTable(MsgId), FOREIGN KEY (TagId) REFERENCES TagTable(TagId))");
			
			statement.close();
			
			super.init();
				
		} catch (SQLException e) {
			
			throw new CommunicationException("Couldn't form a connection with the database" + e.getMessage());
			
		}
				
	}
	
	@PreDestroy
	public void close() throws CommunicationException{
		
		try{
			connection.close();
		} catch (SQLException e) {
			
			throw new CommunicationException("Couldn't close connection to the database");
			
		}
		
	}


	@Override
	protected void insertNewTags(String[] tags) throws SQLException {
		
		PreparedStatement pstmt;
		String sql;
		
		try {
			
		    sql = "INSERT OR IGNORE INTO TagTable(Tag) VALUES (?)";
	    	pstmt = connection.prepareStatement(sql);
		    
		    for (String tag : tags){
		    	pstmt.setString(1, tag);
		    	pstmt.addBatch();
		    }
		    
		    pstmt.executeBatch();
		    pstmt.close();
		    
		} catch (SQLException e){
			throw new SQLException("Error adding new tags to table: " + e.getMessage());
		}
		
	}
	
//	@POST
	@Consumes(Constants.MEDIATYPE_PROTOBUF)
	@Override
	public boolean postMessage(BulletinBoardMessage msg) throws CommunicationException {
		return super.postMessage(msg);
	}
	
//	@GET
	@Consumes(Constants.MEDIATYPE_PROTOBUF)
	@Produces(Constants.MEDIATYPE_PROTOBUF)
	@Override
	public BulletinBoardMessageList readMessages(MessageFilterList filterList) {
		return super.readMessages(filterList);
	}
	
	@GET
	public String test(){
		return "hello";
	}

}
